        ;
        ; Motor de sprites para ZX Spectrum
        ;
        ; por Oscar Toledo Gutirrez
        ;
        ; (c) Copyright 2011 Oscar Toledo Gutirrez
        ;
        ; Creacin: 01-ago-2011. Revisin 3
        ;

        ;
        ; El ZX Spectrum es taaan lento...
        ;
        ; Con este cdigo se pueden visualizar hasta 8 sprites en pantalla,
        ; a partir de la lnea 32 se pueden poner 2 en la misma lnea.
        ; a partir de la lnea 64 se pueden poner 4 en la misma lnea.
        ;
        ; Para mejores resultados es necesario ordenar los sprites por
        ; coordenada Y, primero los que tengan la coordenada Y ms baja.
        ;

        org $8000,$ffff

inicio:
        di
        ld sp,$ffff
        ;
        ; Limpia el bitmap 256x192
        ;
        ld hl,$4000
        ld bc,$1800
_0:     ld (hl),0
        inc hl
        dec bc
        ld a,b
        or c
        jp nz,_0
        ;
        ; Inicia tabla de color 32x24 a color gris con fondo negro
        ;
        ld bc,$0300
_1:     ld (hl),$07
        inc hl
        dec bc
        ld a,b
        or c
        jp nz,_1
        ;
        ; Tabla precalculada para correr un byte por hasta 7 bits a la der.
        ;
        ld hl,$6000
        ld bc,$0800
_4:     push bc
        ld b,$00
_5:     push bc
        ld a,c
        or a
        ld a,b
        ld b,0
        jr z,_6
_7:     rra
        rr b
        dec c
        jr nz,_7
_6:     ld (hl),a
        inc h
        ld (hl),b
        dec h
        inc hl
        pop bc
        inc b
        jr nz,_5
        inc h
        pop bc
        inc c
        djnz _4
        ;
        ; Tabla precalculada para posicin Y de pixel en pantalla
        ;
        ld hl,$7000
        ld bc,$c000
_8:     ld a,c
        and $07
        ld d,a
        ld a,c
        and $c0
        rrca
        scf
        rra
        rrca
        or d
        ld d,a
        ld a,c
        rlca
        rlca
        and $e0
        ld e,a
        ld (hl),e
        inc h
        ld (hl),d
        dec h
        inc l
        inc c
        djnz _8
        call limpia_sprites
        call motor_inicia
_2:     ld hl,sprites
        ld b,11
        ld c,$00
_3:     ld a,c
        srl a
        ld d,a
        srl a
        add a,d
        add a,$30
        ld (hl),a
        inc hl
        ld a,(ticks)
        add a,c
        srl a
        ld (hl),a
        inc hl
        ld a,(ticks)
        and $0c
        rrca
        rrca
        ld (hl),a
        inc hl
        ld (hl),$0f
        inc hl
        ld a,c
        add a,$10
        ld c,a
        djnz _3
        halt
        jp _2
        
        ;
        ; Limpia los sprites.
        ; Los sprites se conservan buffereados
        ; Esta rutina limpia sprites y sprites_anterior
        ;
limpia_sprites:
        ld hl,sprites
        ld b,128
_1:     ld (hl),$d1
        inc hl
        djnz _1
        ld hl,sprites_anterior
        ld b,96
_2:     ld (hl),$00
        inc hl
        djnz _2
        ret

        ;
        ; Inicia motor de sprites
        ; Toma el control completo del ZX Spectrum y as libera IY.
        ;
motor_inicia:
        di
        ld hl,$fe00
        ld de,$fe01
        ld bc,$0100
        ld (hl),$fd
        ldir
        ld a,$c3
        ld ($fdfd),a
        ld hl,motor_vector_int
        ld ($fdfe),hl
        ld a,$fe
        ld i,a
        im 2
        ei
        ret

        ;
        ; Obtiene las coordenadas en pantalla de un sprite
        ;
    MACRO COOR_PIXEL
        ld c,a
        ld b,$70
        ld a,(bc)
        ex af,af'
        inc b
        ld a,(bc)
        ld b,a
        ex af,af'
        ld c,a
        inc hl
        ld a,(hl)
        rrca
        rrca
        rrca
        and $1f
        or c
        ld c,a
    ENDM

        ;
        ; Interrupcin
        ;
motor_vector_int:
        push bc
        push de
        push hl
        push af
        in a,($fe)
        ld a,$01        ; Azul
        out ($fe),a
        call motor_quita_sprites
        ld a,$02        ; Rojo
        out ($fe),a
        call motor_pone_sprites
        ld a,$03        ; Morado
        out ($fe),a
        ld hl,sprites
        ld de,sprites_anterior
_1:     push hl
        ld a,(hl)
        inc a
        cp $c0
        jp c,_2
        ld bc,0
        jp _3

_2:     COOR_PIXEL
_3:     ld a,c
        ld (de),a
        inc de
        ld a,b
        ld (de),a
        inc de
        pop hl
        inc hl
        inc hl
        inc hl
        inc hl
        ld a,l
        cp 255 and (sprites+11*4)
        jr nz,_1
        ld a,$07        ; Gris
        out ($fe),a
        ld hl,(ticks)
        inc hl
        ld (ticks),hl
        pop af
        pop hl
        pop de
        pop bc
        ei
        reti

        ;
        ; Quita los sprites
        ;
motor_quita_sprites:
        ld hl,sprites_anterior
_1:     ld e,(hl)
        inc hl
        ld d,(hl)
        inc hl
        ld a,d
        or e
        call nz,motor_restaura_zona
        ld a,l
        cp 255 and (sprites_anterior+11*2) 
        jp nz,_1
        ret

        ;
        ; Dibuja un byte corrido por hasta 7 pixeles
        ;
    MACRO PIXEL_BYTE
        ld a,(de)
        ld c,a
        ld a,(bc)
        or (hl)
        ld (hl),a
        inc de
        inc b
        inc l
        ld a,(bc)
        or (hl)
        ld (hl),a
        dec b
        dec l
        inc h
        ld a,h
        and 7
        jp nz,_4
        ld a,l
        add a,$20
        ld l,a
        jp c,_4
        ld a,h
        sub 8
        ld h,a
_4:     
    ENDM

        ;
        ; Pone los sprites
        ;
motor_pone_sprites:
        ld hl,sprites
_1:     ld a,(hl)
        inc a
        cp $c0
        jp nc,_2
        push hl
        COOR_PIXEL
        ;
        ; Pone un sprite
        ;
        push bc
        ld a,(hl)
        and $07
        add a,a
        or $60
        ld b,a
        inc hl
        ld a,(hl)       ; Figura sprite
        ld l,0
        rra
        rr l
        rra
        rr l
        rra
        rr l
        ld h,a
        ld de,sprites_heroes
        add hl,de
        ex de,hl
        pop hl
        ;
        ; Pone lado izquierdo
        ;
        push hl
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        pop hl
        ;
        ; Pone lado derecho
        ;
        inc l
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE
        PIXEL_BYTE

        pop hl
_2:     inc hl
        inc hl
        inc hl
        inc hl
        ld a,l
        cp 255 and (sprites+11*4) 
        jp nz,_1
        ret

        ;
        ; Restaura la zona debajo de un sprite
        ; HL = Ap. a memoria de video
        ;
motor_restaura_zona:
        ld b,16
_0:     xor a
        ld (de),a
        inc e
        ld (de),a
        inc e
        ld (de),a
        dec b
        ret z
        dec e
        dec e
        inc d
        ld a,d
        and 7
        jp nz,_0
        ld a,e
        add a,$20
        ld e,a
        jp c,_0
        ld a,d
        sub 8
        ld d,a
        jp _0

        include "sprites.asm"

sprites:
        ds 32*4
sprites_anterior:
        ds 32*2
ticks:  ds 2

        end inicio

